<template>
  <div class="container">
    <authheader :items="menuTips"></authheader>
    <a-card ref="account" class="general-card">
      <div class="table-page-search-wrapper">
        <div class="search-wrapper">
          <div>
            <a-input v-model="formModel.name" :style="{ width: '320px' }" :placeholder="$t('请输入')" />
          </div>
          <div>
            <a-button type="primary" style="margin-right: 20px; margin-left: 10px" @click="search">
              <template #icon>
                <icon-search />
              </template>
              {{ $t('searchTable.form.search') }}
            </a-button>
            <a-button @click="reset">
              <template #icon>
                <icon-refresh />
              </template>
              {{ $t('searchTable.form.reset') }}
            </a-button>
          </div>
        </div>
        <div class="search-wrapper">
          <div>
            <a-space>
              <a-button type="primary" :align="'right'" @click="operation(0)">+ 新建用户组</a-button>
            </a-space>
          </div>
          <div class="wrapper-icon">
            <a-tooltip :content="$t('searchTable.actions.refresh')">
              <div class="action-icon" @click="search">
                <icon-refresh size="18" />
              </div>
            </a-tooltip>
            <a-dropdown @select="handleSelectDensity">
              <a-tooltip :content="$t('searchTable.actions.density')">
                <div class="action-icon">
                  <icon-line-height size="18" />
                </div>
              </a-tooltip>
              <template #content>
                <a-doption v-for="item in densityList" :key="item.value" :value="item.value"
                  :class="{ active: item.value === size }">
                  <span>{{ item.name }}</span>
                </a-doption>
              </template>
            </a-dropdown>
          </div>
        </div>
      </div>
      <a-table row-key="id" :loading="loading" :pagination="pagination" :columns="columns" :data="renderData"
        :bordered="false" :size="size" @page-change="onPageChange">
        <template #index="{ rowIndex }">
          {{ rowIndex + 1 + (pagination.current - 1) * pagination.pageSize }}
        </template>
        <template #status="{ record }">
          <a-switch v-model="record.status" checked-value="1" unchecked-value="0"
            @change="statusChange(record.status, record)" />
        </template>
        <template #operations="{ record }">
          <a-space>
            <a-button type="outline" @click="operation(2, record)">编辑</a-button>
            <a-button type="dashed" status="success" @click="operation(1, record)">成员管理</a-button>
            <a-button type="dashed" status="warning" @click="operation(4, record)">权限管理</a-button>
            <a-popconfirm content="请确认是否删除？" type="success" @ok="operation(3, record)">
              <a-button type="outline" status="danger">删除</a-button>
            </a-popconfirm>
          </a-space>
        </template>
      </a-table>
    </a-card>
    <a-modal v-model:visible="visible" :title="editModalTitle" width="30%" @cancel="handleCancel(1)" @ok="editHandleOk">
      <a-form ref="formRef" :model="editform" auto-label-width>
        <a-row>
          <a-form-item field="loginName" label="用户组名称名称:" :rules="[{ required: true, message: '用户名必填' }]">
            <a-input v-model="editform.loginName" />
          </a-form-item>
        </a-row>
        <a-row>
          <a-form-item field="userName" label="用户组说明:">
            <a-textarea v-model="editform.userName" allow-clear />
          </a-form-item>
        </a-row>
      </a-form>
    </a-modal>
    <a-modal v-model:visible="memberVisible" width="50%" title="用户组成员管理" @cancel="handleCancel(2)"
      @ok="editDeptHandleOk">
      <div :style="{ display: 'flex', justifyContent: 'center' }">
        <!-- TODO默认值回显 -->
        <a-transfer show-search :data="memberList" :default-value="selectedMemberList" :source-input-search-props="{
          placeholder: '请输入用户名搜索'
        }" :target-input-search-props="{
          placeholder: '请输入用户名搜索'
        }">
          <template #source-title>可选列表</template>
          <template #target-title>已选列表</template>
        </a-transfer>
      </div>
    </a-modal>
    <a-modal v-if="resourcevisible" v-model:visible="resourcevisible" width="50%" title="权限管理" ok-text="关闭"
      hide-cancel="true" @ok="handleCancel(3)">
      <div :style="{ 'display': 'flex', 'flex-direction': 'column' }">
        <a-card :style="{
          'width': '100%',
          'height': '40%',
          'overflow-y': 'auto',
          'margin': '1px',
        }" class="card-demo" title="用户所有权限" hoverable>

          <a-space wrap>
            菜单功能：
            <!-- TODO默认值回显 -->
            <a-checkbox-group :default-value="['1']">
              <a-checkbox v-for="(item, index) of checkStrictlyMenu" :value="item.menuId" :key="item.menuId">
                {{ item.menuName }}</a-checkbox>
            </a-checkbox-group>
          </a-space>
          <a-divider />
          <a-space wrap>
            知识库:
            <!-- TODO默认值回显 -->
            <a-checkbox-group :default-value="['1']">
              <a-checkbox v-for="(item, index) of checkStrictlyKnowledge" :value="item.menuId" :key="item.knowledgeId">
                {{ item.knowledgeName }}</a-checkbox>
            </a-checkbox-group>
          </a-space>
          <a-divider />
          <a-space wrap>
            智能体:
            <!-- TODO默认值回显 -->
            <a-checkbox-group :default-value="['1']">
              <a-checkbox v-for="(item, index) of checkStrictlyDialog" :value="item.dialogId" :key="item.menuId">
                {{ item.dialogName }}</a-checkbox>
            </a-checkbox-group>
            <!-- TODO默认值回显 -->
            <a-checkbox-group :default-value="['1']">
              <a-checkbox v-for="(item, index) of checkStrictlyAgent" :value="item.agentId" :key="item.menuId">
                {{ item.agentName }}</a-checkbox>
            </a-checkbox-group>
          </a-space>
        </a-card>
      </div>
    </a-modal>
  </div>
</template>

<script lang="ts" setup>
import { computed, reactive, ref, h, onMounted } from 'vue';
import { useI18n } from 'vue-i18n';
import useLoading from '@/hooks/loading';
import { Pagination } from '@/types/global';
import type { TableColumnData } from '@arco-design/web-vue/es/table/interface';
import {
  DialogList,
  KnowledgeList,
  OrganizationList,
  ResourceList,
  Role,
  RoleList,
  User,
  UserAdd,
  UserChangePwd,
  UserDelete,
  UserEdit,
  UserList,
  Userstatus,
} from '@/api/authority';
import { Modal } from '@arco-design/web-vue';
import Authheader from '@/views/authority/components/authheader.vue';
import { create } from 'lodash';
import { queryCanvasList } from '@/api/Agent';

const memberList = ref([
  {
    label: '张三',
    value: 'zhangsan',
  },
  {
    label: ' 李四',
    value: 'lisi',
  },
  {
    label: '张三',
    value: 'zhangsan',
  },
]);
const selectedMemberList = ref(['zhangsan'])

const roles = ref([]);

const treeData = ref([]);
const checkedKeys = ref([]);
const expandKdys = ref([]);
const checkStrictly = ref([]);
const checkStrictlyMenu = ref([]);
const checkStrictlyKnowledge = ref([]);
const checkStrictlyDialog = ref([]);
const checkStrictlyAgent = ref([]);
const checkStrictlyDeptRole = ref([]);
const formRef = ref();

const menuTips = ref(['权限管理', '用户组']);
type SizeProps = 'mini' | 'small' | 'medium' | 'large';
const account = ref(null);
const generateFormModel = () => {
  return {
    name: '',
  };
};
const showLine = ref(true);
const { loading, setLoading } = useLoading(true);
const { t } = useI18n();
const editModalTitle = ref('新增');
const renderData = ref<User[]>([]);
const formModel = ref(generateFormModel());
const editform = ref<User>({
  createTime: '',
  dept: undefined,
  deptName: '',
  email: '',
  nickName: '',
  phoneNumber: '',
  status: '',
  userId: '',
  loginName: '',
  userName: '',
  psw: '',
  role: [],
});

const size = ref<SizeProps>('medium');
const visible = ref(false);
const memberVisible = ref(false);
const resourcevisible = ref(false);
const selectUser = ref({});

const loadRole = async () => {
  await RoleList(null).then((res) => {
    roles.value = res.rows;
  });
};

const roleChange = (val) => {
  editform.value.role = val;
};

const onCheck = (newCheckedKeys, event) => {
  const o = { deptId: event.node.deptId, deptName: event.node.deptName };
  if (event.checked) {
    event.node.roles.forEach((val) => {
      checkStrictlyDeptRole.value.push(val);
    });
    checkStrictly.value.push(o);
  } else {
    checkStrictly.value.forEach((val, idx, array) => {
      // val: 当前值
      if (val.deptId == event.node.deptId) {
        checkStrictly.value.splice(idx, 1);
        checkStrictlyDeptRole.value.forEach((val2, idx2, array2) => {
          if (val2.deptId == event.node.deptId) {
            checkStrictlyDeptRole.value.splice(idx2, 1);
          }
        });
        return true;
      }
    });
  }
};

const basePagination: Pagination = {
  current: 1,
  pageSize: 15,
};
const pagination = reactive({
  ...basePagination,
});

const densityList = computed(() => [
  {
    name: t('searchTable.size.mini'),
    value: 'mini',
  },
  {
    name: t('searchTable.size.small'),
    value: 'small',
  },
  {
    name: t('searchTable.size.medium'),
    value: 'medium',
  },
  {
    name: t('searchTable.size.large'),
    value: 'large',
  },
]);
const columns = computed<TableColumnData[]>(() => [
  {
    title: t('序号'),
    dataIndex: 'index',
    slotName: 'index',
  },
  // TODO 以下需要字段对接
  {
    title: t('用户组名称'),
    dataIndex: 'loginName',
  },
  {
    title: t('用户组成员'),
    dataIndex: 'roleName',
  },
  {
    title: t('用户组说明'),
    dataIndex: 'deptName',
    slotName: 'deptName',
  },
  {
    title: t('创建时间'),
    dataIndex: 'deptName',
    slotName: 'deptName',
  },
  {
    title: t('状态'),
    dataIndex: 'status',
    slotName: 'status',
  },
  {
    title: t('searchTable.columns.operations'),
    dataIndex: 'operations',
    slotName: 'operations',
  },
]);

const statusChange = async (value, record) => {
  if (record?.dept) {
    await Userstatus(record.userId, value).then((res) => { });
  } else {
    record.status = '0';
    Modal.warning({
      title: '提示',
      content: '如想启用用户，需进行部门配置.',
    });
  }
};

const handleCancel = (type) => {
  if (type == 1) {
    visible.value = false;
  }
  if (type == 2) {
    memberVisible.value = false;
  }
};

const editDeptHandleOk = async () => {
  const depts: Array = [];
  const user: User = { userId: selectUser.value.userId };
  checkStrictly.value.forEach((val) => {
    depts.push(val.deptId);
  });
  user.dept = depts;
  await UserEdit(user).then((res) => {
    fetchData();
  });
};

const cb = async (err) => {
  if (err) {
    visible.value = true;
  } else {
    let uuu;
    if (editform.value.userId.length > 0) {
      await UserEdit({
        ...editform.value,
      } as unknown as User).then((res) => {
        fetchData();
        uuu = res.data;
      });
    } else {
      await UserAdd({
        ...editform.value,
      } as unknown as User).then((res) => {
        fetchData();
        uuu = res.data;
      });
    }
    if (!uuu.dept) {
      memberVisible.value = true;
      checkedKeys.value = [];
      expandKdys.value = [];
      checkStrictly.value = [];
      selectUser.value = uuu;
    }
  }
};
const editHandleOk = () => {
  formRef.value.validate(cb);
};
const operation = async (t, record) => {
  if (t == 0) {
    editModalTitle.value = '新增';
    visible.value = true;
    editform.value.userId = '';
    editform.value.userName = '';
    editform.value.loginName = '';
    editform.value.nickName = '';
    editform.value.email = '';
    editform.value.phoneNumber = '';
    editform.value.role = '';
  }
  // 成员管理
  if (t == 1) {
    memberVisible.value = true;
  }
  // 编辑
  if (t == 2) {
    visible.value = true;
    formRef.value?.resetFields();
    editform.value.role = [];
    editModalTitle.value = '编辑';
    editform.value.userId = record.userId;
    editform.value.userName = record.userName;
    editform.value.loginName = record.loginName;
    editform.value.email = record.email;
    editform.value.phoneNumber = record.phoneNumber;
    if (record.roles?.length > 0) {
      // 遍历record.roles
      record.roles.forEach((val) => {
        editform.value.role.push(val.roleId);
      });
    }
  }
  // 删除
  if (t == 3) {
    await UserDelete(record.userId).then((res) => {
      if (res.code == 200) {
        fetchData();
      }
    });
  }
  // 权限
  if (t == 4) {
    resourcevisible.value = true;
    checkStrictlyMenu.value = [];
    checkStrictlyKnowledge.value = [];
    checkStrictlyDialog.value = [];
    checkStrictlyAgent.value = [];
    selectUser.value = record;

    let agents;
    if (record.agents) {
      agents = record.agents;
    } else {
      agents = record.roles ? record.roles[0].agents : null;
    }
    if (agents) {
      agents.forEach((val) => {
        checkStrictlyAgent.value.push({
          agentId: val.id,
          agentName: val.title,
        });
      });
    }
    let resources;
    if (record.resources) {
      resources = record.resources;
    } else {
      resources = record.roles ? record.roles[0].resources : null;
    }
    if (resources) {
      resources.forEach((val) => {
        checkStrictlyMenu.value.push({
          menuId: val.menuId,
          menuName: val.menuName,
        });
      });
    }
    let knowledges;
    if (record.knowledges) {
      knowledges = record.knowledges;
    } else {
      knowledges = record.roles ? record.roles[0].knowledges : null;
    }
    if (knowledges) {
      knowledges.forEach((val) => {
        checkStrictlyKnowledge.value.push({
          knowledgeId: val.id,
          knowledgeName: val.name,
        });
      });
    }
    const dialogs = record.roles ? record.roles[0].dialogs : null;
    if (dialogs) {
      dialogs.forEach((val) => {
        checkStrictlyDialog.value.push({
          dialogId: val.id,
          dialogName: val.name,
        });
      });
    }
  }
};

const fetchData = async (
  params: Pagination = { current: 1, pageSize: 20 }
) => {
  setLoading(true);
  try {
    await UserList(params).then((res) => {
      for (const user of res.rows) {
        if (user.dept) {
          for (const d of user.dept) {
            if (user.deptName) {
              user.deptName += `${d.deptName},`;
            } else {
              user.deptName = `${d.deptName},`;
            }
          }
        }
        if (user.roles) {
          for (const r of user.roles) {
            if (user.roleName) {
              user.roleName += `${r.roleName},`;
            } else {
              user.roleName = `${r.roleName},`;
            }
          }
        }
      }
      renderData.value = res.rows;
      console.log(renderData);
      pagination.current = params.current;
      pagination.total = res.total;
    });
  } catch (err) {
    // you can report use errorHandler or other
  } finally {
    setLoading(false);
  }
};

const search = () => {
  fetchData({
    ...basePagination,
    ...formModel.value,
  } as unknown as Pagination);
};

const onPageChange = (current: number) => {
  fetchData({ ...basePagination, current });
};

const OrganizationData = async (key) => {
  await OrganizationList(key).then((res) => {
    treeData.value = [...res.rows];
  });
};

fetchData();
OrganizationData('');
loadRole();

const reset = () => {
  formModel.value = generateFormModel();
};

const handleSelectDensity = (
  val: string | number | Record<string, any> | undefined,
  e: Event
) => {
  size.value = val as SizeProps;
};
</script>

<style scoped lang="less">
.card-demo {
  width: 460px;
  margin-left: 24px;
  transition-property: all;
}

.card-demo:hover {
  transform: translateY(-4px);
}

.table-page-search-wrapper {
  padding-top: 10px;
  display: flex;
  justify-content: space-between;
  align-items: center;
  margin-bottom: 20px;
  padding-bottom: 10px;
  border-bottom: 1px solid #e8e8e8;

  .search-wrapper {
    display: flex;

    .wrapper-icon {
      display: flex;
      align-items: center;
      margin-left: 40px;
      // margin-right: 40px;
    }
  }
}

.space_select_all {
  display: flex;
  justify-content: end;
  margin-bottom: 15px;
}

.table_box {
  display: flex;

  border: 1px solid #e8e8e8;
  border-bottom: none;

  .row1_clo {
    min-width: 200px;
    display: flex;
    padding-left: 20px;
    border-right: 1px solid #e8e8e8;
  }

  .table_row1 {
    width: 100%;

    .row1_list {
      display: flex;
      min-height: 40px;
      border-bottom: 1px solid #e8e8e8;
      // line-height: 40px;
    }
  }

  .row1_clo2 {
    width: 100%;
    border-top: none;

    .row2_clo2_1 {
      display: flex;
      width: 100%;
      min-height: 40px;
      line-height: 40px;
      border-bottom: 1px solid #e8e8e8;

      .row1_clo2_1 {
        border-right: 1px solid #e8e8e8;
        min-width: 310px;
        padding-left: 20px;
      }

      .row1_clo2_2 {
        border-top: none;
        border-right: none;
        width: 100%;
        display: flex;
        flex-wrap: wrap;

        div {
          margin-left: 20px;
        }
      }
    }
  }

  .row2_clo2_1:last-child {
    border-bottom: none;
  }
}

:deep(.arco-transfer-view) {
  width: 250px;
  height: 400px
}
</style>

<style lang="less">
.ant-table-wrapper {
  .ant-table-tbody {
    tr {
      td {
        .ant-table-row-cell-break-word {
          .ant-table-column-sorter {
            display: none;
          }
        }
      }
    }
  }

  .search-wrapper {
    display: flex;

    .wrapper-icon {
      display: flex;
      align-items: center;
      margin-left: 40px;
      // margin-right: 40px;
    }
  }
}
</style>

<style lang="less">
.ant-table-wrapper {
  .ant-table-tbody {
    tr {
      td {
        .ant-table-row-cell-break-word {
          .ant-table-column-sorter {
            display: none;
          }
        }
      }
    }
  }
}
</style>

<style lang="less" scoped>
.table-page-search-wrapper {
  .ant-form-inline {
    :deep(.ant-form-item) {
      display: flex;
    }
  }
}
</style>
